function dxdt = quadrotor_dynamics(t,x,u,wind, mode,OC)
% dxdt = QUADROTOR_DYNAMICS(t,x,u,wind,mode,OC)
% 
% Generates the ODEs governing the quadrotor dynamics described in the
% benchmark paper S. Kaynama and C.J. Tomlin, "Benchmark: Flight envelope 
% protection in autonomous quadrotors," in International Workshop on 
% Applied Verification for Continuous and Hybrid Systems, Part of CPSWeek, 
% Berlin, Germany, April 1517, 2014. (cps-vo.org/group/ARCH) 
% 
% The function can be used in conjunction with an ODE solver (e.g. ode45)
% to simulate a trajectory of the quadrotor.
% 
% INPUTS:   t       time
%           x       state vector in R^12 
%                   [x y z xdot ydot zdot phi theta psi phidot thetadot psidot]'
%           u       control input in R^4 [thrust phiddot thetaddot psiddot]'
%           wind    disturbance input here assumed in R^1, affects xdot ydot
%                   zdot simultaneously, but can be modified as desired
%                   (Default: zero)
%           mode    'nonlinear' or 'linear', if linear the system is
%                   linearized around OC (Default: nonlinear)
%           OC      operating condition [phi theta thrust] for linearization 
%                   (Default: hover)
% 
% OUTPUT:   dxdt    the flow vector
% 
% e.g. [T X] = ode45(@(t,y) quadrotor_dynamics(t,y,[1 0 0 0]',0,'nonlinear'),...
%                       [0 1],[zeros(1,2) 5 zeros(1,9)]');
 

% S. Kaynama, 9/30/2014, kaynama@berkeley.edu


g = 9.81;
dxdt = zeros(12,1);

if nargin < 6, OC = [0 0 g]; end
if nargin < 5, mode = 'nonlinear'; end
if nargin < 4, wind = 0; end
if nargin < 3, error('QUADROTOR_DYNAMICS: Not enough input arguments!'); end

switch mode
    case 'nonlinear'
        dxdt(1) = x(4);
        dxdt(2) = x(5);
        dxdt(3) = x(6);
        
        dxdt(4) = -cos(x(7))*sin(x(8))*u(1) + wind;
        dxdt(5) = -sin(x(7))*u(1) + wind;
        dxdt(6) = -g+cos(x(7))*cos(x(8))*u(1) + wind;
        
        dxdt(7) = x(10);
        dxdt(8) = x(11);
        dxdt(9) = x(12);
        
        dxdt(10) = u(2);
        dxdt(11) = u(3);
        dxdt(12) = u(4);

    case 'linear'
        A = [zeros(3,3) eye(3) zeros(3,6);
             zeros(1,6) OC(3)*sin(OC(1))*sin(OC(2)) ... 
                -OC(3)*cos(OC(1))*cos(OC(2)) zeros(1,4);
             zeros(1,6) -OC(3)*cos(OC(1)) zeros(1,5);
             zeros(1,6) -OC(3)*sin(OC(1))*cos(OC(2)) ... 
                 -OC(3)*cos(OC(1))*sin(OC(2)) zeros(1,4);
             zeros(3,9) eye(3);
             zeros(3,12)];
        
        B = [zeros(3,4);
             -cos(OC(1))*sin(OC(2)) zeros(1,3);
             -sin(OC(1)) zeros(1,3);
             cos(OC(1))*cos(OC(2)) zeros(1,3);
             zeros(3,4);
             zeros(3,1) eye(3)];
             
        G = [zeros(3,1); ones(3,1); zeros(6,1)];
        
        dxdt = A*x + B*u + G*wind;
        
    otherwise
        error('QUADROTOR_DYNAMICS: Mode can only be ''nonlinear'' or ''linear''');
end


end
